package fix

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module DataFlow.ForOrderedTypeSource

ForOrderedType is a special case of Run where it suffices to pass an ordered type T as an argument. A reference to a persistent map is used to hold the memoization table.

Parameters

module P : sig ... end
module G : sig ... end

Signature

Sourcetype variable = G.variable

The type of variables.

Sourcetype property = P.property option

The type of properties.

Sourceval solution : variable -> property

The least solution of the system of monotone equations.