package fix

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Glue.TrivialOrderedTypeSource

TrivialOrderedType equips an arbitrary type with an ordering function, just by using OCaml's built-in generic comparison function.

Parameters

module T : sig ... end

Signature

type t = T.t

The type of the map keys.

Sourceval compare : t -> t -> int

A total ordering function over the keys. This is a two-argument function f such that f e1 e2 is zero if the keys e1 and e2 are equal, f e1 e2 is strictly negative if e1 is smaller than e2, and f e1 e2 is strictly positive if e1 is greater than e2. Example: a suitable ordering function is the generic structural comparison function Stdlib.compare.