package fix

  1. Overview
  2. Docs

Module HashCons.ForHashedTypeSource

ForHashedType is a special case of Make where it suffices to pass a hashed type T as an argument. A hash table is used to hold the memoization table.

Parameters

Signature

Sourcetype data = T.t
Sourceval make : data -> data cell